#!/usr/bin/env python
# -*- coding: utf-8 -*-

import copy
import json
import logging
import os
import platform
import sys
import time
import traceback

if not sys.version_info < (3,):
    unicode = str
    PY3 = True
else:
    bytes = str
    PY3 = False

from utils import template
from utils import variables
from modules import shell
from modules import xml_query

is_windows = True if platform.system().lower() == 'windows' else False
if not is_windows:
    from modules import file_permission

class RunContext:
    def __init__(self):
        self._template = None
        self._variable_manager = variables.VariableManager()
        self._params = dict()
        self._templatesdir = None
        self._module_path = []
        self._ansible_module_params = {}
        self._poll_interval = 1.0
        self._result_lang = 'zh_cn'
        self._simple_result = False
        self._interpreter = '/usr/bin/python'

    def get_mod_params_instance(self):
        '''Get the value of a module parameter'''
        return copy.deepcopy(self._ansible_module_params)

    def set_mod_params_instance(self, params, key, value):
        '''Set the value of a module parameter'''
        params[key] = value

    def set_template(self, dir):
        '''Set template'''
        self._templatesdir = dir
        self._params['templatesdir'] = dir
        self._template = template.Templar(dir)

    def get_template(self):
        '''Get template'''
        return self._template

    def set_parameter(self, key, value):
        '''Set the value of a component parameter'''
        self._params[key] = value

    def get_parameter(self, key):
        '''Get the value of a component parameter'''
        return self._params[key]

    def set_module_path(self, path):
        '''Set module path'''
        if path != None:
            path = path.strip(os.pathsep)
            self._module_path = path.split(os.pathsep)

    def get_module_file(self, name):
        '''Get module path'''
        for p in self._module_path:
            path = os.path.join(p, 'modules', name)
            if os.path.exists(path):
                return path
        raise Exception('Module file %s is not exists in %s' % (name, ':'.join(self._module_path)))

    def set_poll_interval(self, value):
        self._poll_interval = float(value)

    def get_poll_interval(self):
        return self._poll_interval

    def get_extra_vars(self):
        ''' Make sure a clean copy of extra_vars is made '''
        return self._variable_manager.get_extra_vars()

    def set_extra_vars(self, value):
        ''' Make sure to use a clean copy of extra_vars to set the value '''
        self._variable_manager.set_extra_vars(value)

    def set_extra_var(self, key, value):
        '''Set third-party component parameter values, such as MYSQL_BASEDIR'''
        self._variable_manager.set_extra_var(key, value)

    def get_extra_var(self, key):
        '''Get third-party component parameter values, such as MYSQL_BASEDIR'''
        return self._variable_manager.get_extra_var(key)

    def set_extra_vars_with_string(self, string):
        '''Parse input parameter values'''
        value_list = string.split('=')
        if len(value_list) != 2:
            raise Exception('Add variables failed.')
        else:
            self._variable_manager.set_extra_var(value_list[0], value_list[1])

    def sleep(self, value=None):
        '''Sleep time between each command'''
        if value is None:
            time.sleep(float(self._poll_interval))
        else:
            time.sleep(value)

    def invoke_module(self, module, json_params):
        '''Load module'''
        if os.path.exists(module) is False:
            return -1, "Module %s is not exists." % module

        r_num = 0
        value = ''
        try:
            module_name = os.path.split(module)[1]
            module_shortname = os.path.splitext(module_name)[0]
            if module_shortname == 'shell':
                value = shell.run(json_params)
            elif module_shortname == 'file_permission':
                value = file_permission.run(json_params)
            elif module_shortname == 'xml_query':
                value = xml_query.run(json_params)
            else:
                r_num = -1
                value = "Not support module type %s" % module
        except Exception as err:
            r_num = -1
            value = str(err)
            self.log_debug_invoke_result(r_num, value, traceback.format_exc())
        return r_num, value, ''

    def get_result_lang(self):
        '''Get report template language'''
        return self._result_lang

    def set_result_lang(self, value):
        '''Setting the report language'''
        self._result_lang = value

    def get_simple_result(self):
        '''Whether the report template is a simple template'''
        return self._simple_result

    def set_simple_result(self, value):
        '''Whether the report template is a simple template'''
        self._simple_result = value

    def get_template_file(self, file_name):
        '''Get report type'''
        if self._simple_result is True:
            template_path = os.path.join('simple', file_name)
        elif self._result_lang == 'zh_cn':
            template_path = os.path.join('zh_cn', file_name)
        else:
            template_path = os.path.join('en_us', file_name)

        template_path = template_path.replace('\\', '/') if is_windows else template_path
        return template_path

    def log_debug(self, msg):
        '''Print log level'''
        logger = logging.getLogger('playbook')
        logger.debug(msg)

    def log_info(self, msg):
        '''Print log level'''
        logger = logging.getLogger('playbook')
        logger.info(msg)

    def log_error(self, msg):
        '''Print log level'''
        logger = logging.getLogger('playbook')
        logger.error(msg)

    def log_debug_invoke_result(self, re_out, stdout, stderr):
        """Output log information"""
        logger = logging.getLogger('playbook')
        if logger.level >= logging.DEBUG:
            logger.debug('rc = [%s]\nstdout = %s\nstderr = %s\n' % (str(re_out), str(stdout), str(stderr)))

    def log_debug_json(self, msg):
        """LogLevel:DEBUG = 0,INFO = 1,FATEL = 2"""
        logger = logging.getLogger('playbook')
        if logger.level >= logging.DEBUG:
            logger.debug(json.dumps(msg, indent=4))

    def replace_inject_characters(self, old_str):
        '''Remove special characters'''
        if old_str is None:
            return None

        old_str = old_str.replace(',', '')
        old_str = old_str.replace(';', '')
        old_str = old_str.replace('<', '')
        old_str = old_str.replace('>', '')
        old_str = old_str.replace('|', '')
        old_str = old_str.replace('#', '')
        old_str = old_str.replace('&', '')
        old_str = old_str.replace('$', '')
        old_str = old_str.replace('&&', '')
        old_str = old_str.replace('||', '')
        old_str = old_str.replace('>>', '')

        return old_str
